def get_ppa_published_binaries(launchpad, lp_arch_series, ppa):
    ppa_owner, ppa_name = ppa.split("/")
    archive = launchpad.people[ppa_owner].getPPAByName(name=ppa_name)
    # using pocket "Release" when using a PPA ...'
    pocket = "Release"
    sources = archive.getPublishedBinaries(
        exact_match=True,
        pocket=pocket,
        distro_arch_series=lp_arch_series,
        status="Published",
        ordered=True,
    )
    return sources


def get_ppa_sources(launchpad, lp_series, ppa):
    ppa_owner, ppa_name = ppa.split("/")
    archive = launchpad.people[ppa_owner].getPPAByName(name=ppa_name)
    # using pocket "Release" when using a PPA ...'
    pocket = "Release"
    sources = archive.getPublishedSources(
        exact_match=True,
        pocket=pocket,
        distro_series=lp_series,
        status="Published",
    )
    return sources



import os
import sys
import time
from launchpadlib.credentials import RequestTokenAuthorizationEngine
from lazr.restfulclient.errors import HTTPError
from launchpadlib.launchpad import Launchpad
from launchpadlib.credentials import UnencryptedFileCredentialStore

ACCESS_TOKEN_POLL_TIME = 1
WAITING_FOR_USER = """Open this link:
{}
to authorize this program to access Launchpad on your behalf.
Waiting to hear from Launchpad about your decision. . . ."""


class AuthorizeRequestTokenWithConsole(RequestTokenAuthorizationEngine):
    """Authorize a token in a server environment (with no browser).

    Print a link for the user to copy-and-paste into his/her browser
    for authentication.
    """

    def __init__(self, *args, **kwargs):
        # as implemented in AuthorizeRequestTokenWithBrowser
        kwargs["consumer_name"] = None
        kwargs.pop("allow_access_levels", None)
        super(AuthorizeRequestTokenWithConsole, self).__init__(*args, **kwargs)

    def make_end_user_authorize_token(self, credentials, request_token):
        """Ask the end-user to authorize the token in their browser."""
        authorization_url = self.authorization_url(request_token)
        print(WAITING_FOR_USER.format(authorization_url))
        # if we don't flush we may not see the message
        sys.stdout.flush()
        while credentials.access_token is None:
            time.sleep(ACCESS_TOKEN_POLL_TIME)
            try:
                credentials.exchange_request_token_for_access_token(
                    self.web_root)
                break
            except HTTPError as e:
                if e.response.status == 403:
                    # The user decided not to authorize this
                    # application.
                    raise e
                elif e.response.status == 401:
                    # The user has not made a decision yet.
                    pass
                else:
                    # There was an error accessing the server.
                    raise e


def get_launchpad(launchpadlib_dir=None, lp_credentials_store=None):
    """return a launchpad API class. In case launchpadlib_dir is
    specified used that directory to store launchpadlib cache instead of
    the default"""
    if not lp_credentials_store:
        creds_prefix = os.environ.get("SNAP_USER_COMMON",
                                      os.path.expanduser("~"))
        store = UnencryptedFileCredentialStore(
            os.path.join(creds_prefix, ".launchpad.credentials")
        )
    else:
        store = UnencryptedFileCredentialStore(lp_credentials_store)
    lp_app = "ubuntu-ppa-package-changelog-diff-generator"
    lp_env = "production"
    lp_version = "devel"

    authorization_engine = AuthorizeRequestTokenWithConsole(lp_env, lp_app)
    return Launchpad.login_with(
        lp_app,
        lp_env,
        credential_store=store,
        authorization_engine=authorization_engine,
        launchpadlib_dir=launchpadlib_dir,
        version=lp_version,
    )



"""Console script for ubuntu_ppa_package_version_report."""
import csv
import os
import sys
import click

from ubuntu_ppa_package_version_report import (
    launchpadagent,
)

from ubuntu_ppa_package_version_report.ubuntu_ppa_package_version_report \
    import (
        get_ppa_published_binaries,
        get_ppa_sources,
    )


@click.command()
@click.option(
    "--lp-credentials-store",
    envvar="LP_CREDENTIALS_STORE",
    required=False,
    help="An optional path to an already configured launchpad "
         "credentials store.",
    default=None,
    type=click.STRING,
)
@click.option(
    "--ppa",
    "ppas",
    help="PPA (Personal Package Archive) to use as the final "
         "source of the diff",
    required=True,
    multiple=True,
    type=click.STRING,
)
@click.option(
    "--series",
    help='the Ubuntu series eg. "20.04" or "focal"',
    required=True,
    type=click.STRING,
)
@click.option(
    "--binary-architecture",
    help="The architecture of the binary packages you are querying. "
    "The default is amd64",
    default="amd64",
    show_default=True,
    type=click.STRING,
)
@click.option(
    "--binary-versions",
    is_flag=True,
    default=False,
    help="Report on the binary versions as well as source package "
         "versions",
)
@click.option(
    "--csv-output",
    is_flag=True,
    default=False,
    help="Output in CSV format. This does not apply for binary package versions",
)
def main(lp_credentials_store, ppas, series, binary_architecture,
         binary_versions, csv_output):
    """Console script for ubuntu_ppa_package_version_report."""
    cachedir_prefix = os.environ.get("SNAP_USER_COMMON", "/tmp")
    launchpad_cachedir = os.path.join(
        f"{cachedir_prefix}/ubuntu_ppa_package_version_report/.launchpadlib"
    )
    launchpad = launchpadagent.get_launchpad(
        launchpadlib_dir=launchpad_cachedir,
        lp_credentials_store=lp_credentials_store
    )
    ubuntu = launchpad.distributions["ubuntu"]
    lp_series = ubuntu.getSeries(name_or_version=series)
    lp_arch_series = lp_series.getDistroArchSeries(archtag=binary_architecture)
    if csv_output:
        csv_stdout_writer = csv.writer(sys.stdout)
        csv_stdout_writer.writerow(
            [
                "suite",
                "ppa",
                "package",
                "version",
            ]
        )
    for _ppa in ppas:
        _ppa_name = _ppa.split("/")[1]
        if not csv_output:
            click.echo(_ppa_name)
        _sources = get_ppa_sources(launchpad, lp_series, _ppa)

        for _source in _sources:
            _source_package_name = _source.source_package_name
            _source_package_version = _source.source_package_version
            if not csv_output:
                click.echo(f"\t{_source_package_name} - {_source_package_version}")
            else:
                csv_stdout_writer.writerow(
                    [
                        series,
                        _ppa_name,
                        _source_package_name,
                        _source_package_version,
                    ]
                )

        if binary_versions:
            if not csv_output:
                click.echo("\tPublished binaries:")
            _binaries = get_ppa_published_binaries(launchpad, lp_arch_series,
                                                   _ppa)
            # get current version for  package in the PPA
            for _binary in _binaries:
                _binary_package_name = _binary.binary_package_name
                _binary_package_version = _binary.binary_package_version
                if not csv_output:
                    click.echo(f"\t\t{_binary_package_name} - "
                           f"{_binary_package_version}")


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['Click>=7.0', 'launchpadlib']

test_requirements = [ ]

setup(
    author="Philip Roche",
    author_email='phil.roche@canonical.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Utility to Generate a report on the versions of packages in specified PPAs",
    entry_points={
        'console_scripts': [
            'ubuntu_ppa_package_version_report=ubuntu_ppa_package_version_report.cli:main',
        ],
    },
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='ubuntu_ppa_package_version_report',
    name='ubuntu_ppa_package_version_report',
    packages=find_packages(include=['ubuntu_ppa_package_version_report', 'ubuntu_ppa_package_version_report.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/canonical/ubuntu_ppa_package_version_report',
    version='0.0.4',
    zip_safe=False,
)
